"""Пакет для работы с диском"""
import shutil
import logging

from ftplib import Error
from os.path import join, isdir, islink, getsize, getmtime, exists

from .storage import Storage, FileData, EntryType


class Disk(Storage):
    """Disk класс взаимодействия с диском"""
    def _copy(self, source: str, destination: str) -> bool:
        if not exists(source):
            return False

        if isdir(source):
            shutil.copytree(source, destination)
        else:
            logging.info(shutil.copy2(source, destination))
        return True

    def files(self, path: str) -> ([FileData], bool):
        """files получает информацию о файлах в директории path"""
        files = []
        if not isdir(path):
            return files, False
        try:
            for filename in listdir(path):
                file_path = join(path, filename)
                if isdir(file_path):
                    file_type = EntryType.FOLDER
                if islink(file_path):
                    file_type = EntryType.LINK
                else:
                    file_type = EntryType.FILE
                files.append(FileData(filename, getsize(file_path), file_type, getmtime(file_path)))
            return files, True
        except (FileNotFoundError, Error):
            return files, False

    def upload(self, source: str, destination: str) -> bool:
        """upload копирует source в destination"""
        try:
            return self._copy(source, destination)
        except FileNotFoundError:
            return False

    def download(self, source: str, destination: str) -> bool:
        """download копирует из source в destination"""
        try:
            return self._copy(source, destination)
        except FileNotFoundError:
            return False



"""Сущности для работы с хранилищами"""
from dataclasses import dataclass

class Type:
    """Класс типа хранилища"""
    DISK = "disk"
    FTP = "ftp"
    S3 = "s3"

@dataclass
class Key:
    """Класс доступа к хранилищу"""
    url: str
    username: str
    password: str

@dataclass
class Resource:
    """Класс доступа к ресурсу хранилища"""
    type: Type
    key: Key
    path: str


"""Пакет для работы с S3 хранилищем"""
import boto3
from os.path import basename, join
from os import listdir

from botocore.client import Config

from .storage import Storage, FileData, EntryType


class S3(Storage):
    """S3 класс для работы с S3 хранилищем"""
    def __init__(
        self, url: str = "http://localhost:9000", key_id: str = "admin", secret_key: str = "admin"
    ) -> None:
        self._s3 = boto3.resource(
            "s3",
            endpoint_url=url,
            aws_access_key_id=key_id,
            aws_secret_access_key=secret_key,
            config=Config(signature_version="s3v4"),
        )

    def files(self, path: str) -> ([FileData], bool):
        """files получает информацию о файлах в директории path в S3 хранилище"""
        files = []
        try:
            for file in self._s3.Bucket(path).objects.all():
                files.append(FileData(name=file.key, etype=EntryType.FILE))
            return files, True
        except (
            FileNotFoundError,
            boto3.exceptions.botocore.exceptions.ClientError,
            boto3.exceptions.botocore.exceptions.ParamValidationError,
        ):
            return files, False

    def upload(self, source: str, destination: str) -> bool:
        """upload загружает файл source в S3 хранилище"""
        filename = basename(source)
        try:
            bucket_name, path = destination.split("/", 1)
            bucket = self._s3.Bucket(bucket_name)
            if filename == "":
                for filename in listdir(source):
                    bucket.upload_file(join(source, filename), join(path, filename))
            else:
                if basename(destination) == "":
                    destination = join(destination, filename)
                bucket.upload_file(path, destination)
            return True
        except (FileNotFoundError, boto3.exceptions.botocore.exceptions.ClientError):
            return False

    def download(self, source: str, destination: str) -> bool:
        """download скачивает файл source из S3 хранилища"""
        filename = basename(source)
        try:
            bucket_name, path = source.split("/", 1)
            bucket = self._s3.Bucket(bucket_name)
            if filename == "":
                for s3_object in bucket.objects.filter(Prefix=path):
                    bucket.download_file(s3_object.key, join(destination, basename(s3_object.key)))
            else:
                if basename(destination) == "":
                    destination = join(destination, filename)
                bucket.download_file(path, destination)
            return True
        except (
            boto3.exceptions.botocore.exceptions.ClientError,
            boto3.exceptions.botocore.exceptions.ParamValidationError,
        ):
            return False
